// Copyright 2006-2007 all rights reserved; see LICENSE file for BSD-style license

package edu.berkeley.sbp;
import edu.berkeley.sbp.util.*;
import edu.berkeley.sbp.Sequence.Pos;
import edu.berkeley.sbp.Sequence.Pos;
import java.util.*;

final class Result implements GraphViz.ToGraphViz {

    private Forest f;
    private Node pred;
    private HashSet<Node> successors = new HashSet<Node>();
    private boolean destroyed = false;
    private int usedByNonDoomedNode = 0;
    private Pos reduction;

    public GSS.Phase phase() { return pred==null ? null : pred.phase(); }
    public Forest getForest() { return f; }
    public Node pred() { return pred; }
    public void addSucc(Node succ) {
        successors.add(succ);
        usedByNonDoomedNode += succ.state().doomed ? 0 : 1;
    }
    public void removeSucc(Node succ) {
        if (!successors.contains(succ)) return;
        successors.remove(succ);
        usedByNonDoomedNode -= succ.state().doomed ? 0 : 1;
        check();
    }

    public boolean usedByAnyNode() { return successors.size() > 0; }
    public boolean usedByNonDoomedNode() { return usedByNonDoomedNode > 0; }

    public String toString() { return super.toString()+"->"+pred(); }

    public void check() { if (successors.size()==0) destroy(); }
    public void destroy() {
        if (destroyed) return;
        if (pred==null) return;  // never destroy the "primordeal" result
        destroyed = true;
        pred.removeSucc(this);
        while(successors.size() > 0)
            for(Node n : successors) {
                removeSucc(n);
                n.removeResult(this);
                break;
            }
    }

    public static void newResult(Forest f, Node pred, Pos reduction, GSS.Phase target) {
        Result r = new Result(f, pred, reduction);
        if (reduction == null) return;
        Parser.Table.State state0 = (Parser.Table.State)pred.state().gotoSetNonTerminals.get(reduction);
        target.newNodeFromReduction(r, state0, reduction);
    }
    public Result(Forest f, Node pred, Pos reduction) {
        this.f = f;
        this.pred = pred;
        this.reduction = reduction;
        if (pred != null) pred.addSucc(this);
    }

    // GraphViz //////////////////////////////////////////////////////////////////////////////

    public GraphViz.Node toGraphViz(GraphViz gv) {
        if (gv.hasNode(this)) return gv.createNode(this);
        GraphViz.Node n = gv.createNode(this);
        n.label = ""+f;
        n.shape = "rectangle";
        if (pred()!=null) n.edge(pred, "");
        n.color = "blue";
        if (phase() != null)
            ((GraphViz.Group)phase().toGraphViz(gv)).add(n);
        return n;
    }
    public boolean isTransparent() { return false; }
    public boolean isHidden() { return false; }

}